@@ -36,7 +36,7 @@ class InitialHyperparameters(GlobalObserver):
 
         super().__init__(**kwargs)
 
-        force_skip = ["samples"]
+        force_skip = ["samples", "gradient_accumulation"]
         skip_hparams = force_skip if skip_hparams is None else skip_hparams + force_skip
         self.skip_hparams = [] if skip_hparams is None else skip_hparams
         self.pad_with = pad_with
