@@ -193,13 +193,14 @@ class Statistic(ABC):
         Processes the tensor cache to build a TensorStatistic model.
         """
 
-        concatenated = torch.cat(self._tensor_cache)
-        self._tensor_cache = []
+        if self._tensor_cache:
+            concatenated = torch.cat(self._tensor_cache)
+            self._tensor_cache = []
 
-        statistics = TensorStatistics.build(
-            tensor=concatenated, skip_statistics=self.skip_statistics, sample_percentage=self.downsample_percent
-        )
-        self._data.append(statistics)  # type: ignore
+            statistics = TensorStatistics.build(
+                tensor=concatenated, skip_statistics=self.skip_statistics, sample_percentage=self.downsample_percent
+            )
+            self._data.append(statistics)  # type: ignore
 
     @staticmethod
     @final
@@ -213,10 +214,10 @@ class Statistic(ABC):
 
         if torch_distributed_utils.is_scheduler_master_rank():
             if isinstance(statistic, torch.Tensor):
-                shape = statistic.shape
+                shape = statistic.view(-1).shape
 
             elif isinstance(statistic, TensorStatistics):
-                shape = statistic.to_tensor().shape
+                shape = statistic.to_tensor().view(-1).shape
 
             elif statistic is not None:
                 shape = torch.tensor([statistic]).shape
@@ -239,23 +240,21 @@ class Statistic(ABC):
         if not torch_distributed_utils.is_distributed():
             return statistic
 
+        shape = self._determine_reduction_shape(statistic=statistic)
+
         if statistic is None:
-            shape = self._determine_reduction_shape(statistic=statistic)
-
-            if shape is None:
-                return statistic
-
-            to_reduce = torch.zeros(shape)
+            to_reduce = torch.zeros(shape, dtype=torch.float64)
 
         elif isinstance(statistic, torch.Tensor):
-            to_reduce = statistic.clone()
+            to_reduce = statistic.clone().to(torch.float64).view(-1)
 
         elif isinstance(statistic, TensorStatistics):
-            to_reduce = statistic.to_tensor()
+            to_reduce = statistic.to_tensor().to(torch.float64).view(-1)
 
         else:
-            to_reduce = torch.tensor([statistic])
+            to_reduce = torch.tensor([statistic], dtype=torch.float64)
 
+        to_reduce = to_reduce.to(torch_distributed_utils.get_local_device())
         dist.reduce(to_reduce, dst=MASTER_SCHEDULER_RANK, op=ReduceOp.SUM)
 
         if not torch_distributed_utils.is_scheduler_master_rank():
@@ -288,23 +287,21 @@ class Statistic(ABC):
             statistic = self._gather(
                 optimizer=optimizer, model=model, parameters=parameters, parameter_group=parameter_group
             )
+
             statistic = self._distributed_reduce(statistic=statistic)
 
-            if not torch_distributed_utils.is_scheduler_master_rank():
-                return
+            if torch_distributed_utils.is_scheduler_master_rank():
+                if isinstance(statistic, torch.Tensor):
+                    statistic = statistic.view(-1)
+                    self._tensor_cache.append(statistic)
 
-            if isinstance(statistic, torch.Tensor):
-                statistic = statistic.view(-1)
-                self._tensor_cache.append(statistic)
+                    if len(self._tensor_cache) >= self.max_cache_size:
+                        self._process_tensor_cache()
 
-                if len(self._tensor_cache) >= self.max_cache_size:
-                    self._process_tensor_cache()
+                elif statistic is not None:
+                    self._data.append(statistic)  # type: ignore
 
-            elif statistic is not None:
-                self._data.append(statistic)  # type: ignore
-
-        if torch_distributed_utils.is_scheduler_master_rank():
-            self._sample_number += 1
+        self._sample_number += 1
 
     @final
     def fetch(self) -> TensorStatistics | float | None:
@@ -406,7 +403,9 @@ class FirstOrderGradients(Statistic):
         :return: None, TensorStatistics model or a float.
         """
 
-        gradients = [p.grad.view(-1) for p in parameters if observation_utils.tensor_on_local_rank(p)]
+        gradients = [
+            p.grad.view(-1) for p in parameters if observation_utils.tensor_on_local_rank(p) and p.grad is not None
+        ]
 
         if not gradients:
             return None
@@ -498,7 +497,9 @@ class SecondOrderGradients(Statistic):
         :return: None, TensorStatistics model or a float.
         """
 
-        fo_gradients = [p.grad.view(-1) for p in parameters if observation_utils.tensor_on_local_rank(p)]
+        fo_gradients = [
+            p.grad.view(-1) for p in parameters if observation_utils.tensor_on_local_rank(p) and p.grad is not None
+        ]
 
         if not fo_gradients:
             return None
